#ifndef UTILS_H
#define UTILS_H

#define DEFAULT_PATH_MERGE_THRESHOLD 0
//paths whose lengths differ by at most this value are considered equally long;
//if you set this to a negative value, then no merging will happen at all,
//leading to exponentially many paths examined even for graphs with unit edges

typedef std::tuple<int, int, double> EDGE_TYPE;




#define MAXN 0 //just an identifier - do not change!
#define MAXQ 1 //just an identifier - do not change!
#define MAXG 2 //just an identifier - do not change!
#define MAXQG 3 //just an identifier - do not change!


struct PAR_MANAGER
{
	union
	{
		int par[4];
		struct
		{
			int n, q, g, qg;
		};
	};

	void* pBegin = NULL;
	void* pEnd;

	void Init(void* p, int nb, const std::vector<int>& memory, int g)
	{
		if(pBegin)
		{
			throw std::runtime_error("Trying to create two instances of GRAPH in parallel!");
		}

		pBegin = p;
		pEnd = ((char*) p) + nb;
		n = memory.size();
		this->g = g;

		q = 0;
		for(int i = 0; i < n; i++)
		{
			q += memory[i];
		}

		qg = 0;
		for(int i = 0; i < g; i++)
		{
			qg += memory[i];
		}
	}

	void InitFinished()
	{
		pBegin = NULL;
	}
}
ParManager;

template<typename T, int S>
struct ARRAY1D
{
	int size = 0;
	T* a = NULL;

	ARRAY1D()
	{
		if(!((this >= ParManager.pBegin) && (this < ParManager.pEnd)))
		{
			throw std::runtime_error("Trying to create an array outside the current instance of GRAPH!");
		}
		
		size = ParManager.par[S];
		a = new T[size]();
	}

	~ARRAY1D()
	{
		delete[] a;
	}

	ARRAY1D& operator = (ARRAY1D& other)
	{
		for(int i = 0; i < size; i++)
		{
			(*this)[i] = other[i];
		}

		return *this;
	}

	T& operator [] (int i)
	{
		return a[i];
	}
};

template<typename T, int S1, int S2>
struct ARRAY2D
{
	int size1 = 0;
	int size2 = 0;
	T* a = NULL;

	ARRAY2D()
	{
		if(!((this >= ParManager.pBegin) && (this < ParManager.pEnd)))
		{
			throw std::runtime_error("Trying to create an array outside the current instance of GRAPH!");
		}
		
		size1 = ParManager.par[S1];
		size2 = ParManager.par[S2];
		a = new T[size1 * size2];
	}

	~ARRAY2D()
	{
		delete[] a;
	}

	ARRAY2D& operator = (ARRAY2D& other)
	{
		for(int i = 0; i < size1; i++)
		{
			for(int j = 0; j < size2; j++)
			{
				(*this)[i][j] = other[i][j];
			}
		}

		return *this;
	}

	T* operator [] (int i)
	{
		return a + (i * size2);
	}
};

template<typename T, int S1, int S2, int S3>
struct ARRAY3D
{
	int size1 = 0;
	int size2 = 0;
	int size3 = 0;
	T* a = NULL;

	ARRAY3D()
	{
		if(!((this >= ParManager.pBegin) && (this < ParManager.pEnd)))
		{
			throw std::runtime_error("Trying to create an array outside the current instance of GRAPH!");
		}
		
		size1 = ParManager.par[S1];
		size2 = ParManager.par[S2];
		size3 = ParManager.par[S3];
		a = new T[size1 * size2 * size3];
	}

	~ARRAY3D()
	{
		delete[] a;
	}

	ARRAY3D& operator = (ARRAY3D& other)
	{
		for(int i = 0; i < size1; i++)
		{
			for(int j = 0; j < size2; j++)
			{
				for(int k = 0; k < size3; k++)
				{
					(*this)[i][j][k] = other[i][j][k];
				}
			}
		}

		return *this;
	}

	struct HELPER
	{
		ARRAY3D* p;
		int i;

		HELPER(ARRAY3D* p, int i): p(p), i(i) {}
		T* operator [] (int j) {return p->a + (((i * p->size2) + j) * p->size3);}
	};

	HELPER operator [] (int i)
	{
		return HELPER(this, i);
	}
};

#endif //UTILS_H
